<?php

namespace Feature;

use EventPortal\AuthPassword\Mail\CreatePassword;
use EventPortal\AuthPassword\Tests\TestCase;
use EventPortal\AuthPassword\Tests\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Mail;

class CreatePasswordTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function receives_email_to_create_password()
    {
        Mail::fake();

        $user = User::create([
            'email' => 'someone@someone.com',
            'password' => 'password',
            'password_token' => 'TOKEN',
            'password_created_at' => null,
        ]);

        Mail::assertSent(CreatePassword::class, static function (CreatePassword $mail)  use ($user) {
            return $mail->hasTo($user->email) && $mail->token === $user->password_token;
        });
    }

    /** @test */
    public function update_password()
    {
        $user = User::create([
            'email' => 'someone@someone.com',
            'password' => 'password',
            'password_token' => 'TOKEN',
            'password_created_at' => null,
        ]);

        $this->post(route('create-password.store', 'TOKEN'), [
            'password' => 'NewPassword1',
            'password_confirmation' => 'NewPassword1',
        ]);

        $this->assertNotNull($user->fresh()->password_created_at);
    }

    /** @test */
    public function password_must_be_eight_characters_long()
    {
        User::create([
            'email' => 'someone@someone.com',
            'password' => 'password',
            'password_token' => 'TOKEN',
            'password_created_at' => null,
        ]);

        $response = $this->post(route('create-password.store', 'TOKEN'), [
            'password' => 'Pass1',
            'password_confirmation' => 'Pass1',
        ]);

        $response->assertSessionHasErrors([
            'password' => 'Your password must be at least eight characters long, contain at least'
                .' one uppercase character, one lowercase character, and one number.',
        ]);
    }

    /** @test */
    public function password_must_be_confirmed()
    {
        User::create([
            'email' => 'someone@someone.com',
            'password' => 'password',
            'password_token' => 'TOKEN',
            'password_created_at' => null,
        ]);

        $response = $this->post(route('create-password.store', 'TOKEN'), [
            'password' => 'Password1',
            'password_confirmation' => 'Password',
        ]);

        $response->assertSessionHasErrors([
            'password' => 'The password confirmation does not match.'
        ]);
    }

    /** @test */
    public function password_must_have_one_uppercase()
    {
        User::create([
            'email' => 'someone@someone.com',
            'password' => 'password',
            'password_token' => 'TOKEN',
            'password_created_at' => null,
        ]);

        $response = $this->post(route('create-password.store', 'TOKEN'), [
            'password' => 'password1',
            'password_confirmation' => 'password',
        ]);

        $response->assertSessionHasErrors([
            'password' => 'Your password must be at least eight characters long, contain at least'
                .' one uppercase character, one lowercase character, and one number.',
        ]);
    }

    /** @test */
    public function password_must_have_one_number()
    {
        User::create([
            'email' => 'someone@someone.com',
            'password' => 'password',
            'password_token' => 'TOKEN',
            'password_created_at' => null,
        ]);

        $response = $this->post(route('create-password.store', 'TOKEN'), [
            'password' => 'Password!',
            'password_confirmation' => 'Password!',
        ]);

        $response->assertSessionHasErrors([
            'password' => 'Your password must be at least eight characters long, contain at least'
                .' one uppercase character, one lowercase character, and one number.',
        ]);
    }
}
